@@ -106,6 +106,11 @@ static int hash(const char *name) {
  */
 }
 
+void blankversion(struct versionrevision *version) {
+  version->epoch= 0;
+  version->version= version->revision= 0;
+}
+
 void blankpackage(struct pkginfo *pigp) {
   pigp->name= 0;
   pigp->status= stat_notinstalled;
@@ -113,6 +118,8 @@ void blankpackage(struct pkginfo *pigp) {
   pigp->want= want_unknown;
   pigp->priority= pri_unknown;
   pigp->section= 0;
+  blankversion(&pigp->configversion);
+  pigp->files= 0;
   pigp->installed.valid= 0;
   pigp->available.valid= 0;
   pigp->clientdata= 0;
@@ -122,7 +129,9 @@ void blankpackageperfile(struct pkginfoperfile *pifp) {
   pifp->essential= 0;
   pifp->depends= 0;
   pifp->depended= 0;
-  pifp->description= pifp->maintainer= pifp->version= pifp->revision= 0;
+  pifp->description= pifp->maintainer= pifp->source= 0;
+  pifp->architecture= 0;
+  blankversion(&pifp->version);
   pifp->conffiles= 0;
   pifp->arbs= 0;
   pifp->valid= 1;
@@ -130,27 +139,30 @@ void blankpackageperfile(struct pkginfoperfile *pifp) {
 
 static int nes(const char *s) { return s && *s; }
 
-int informativeperfile(struct pkginfoperfile *info) {
-  /* Used by dselect as an aid to decide whether to display things. */
+int informative(struct pkginfo *pkg, struct pkginfoperfile *info) {
+  /* Used by dselect and dpkg query options as an aid to decide
+   * whether to display things, and by dump to decide whether to write them
+   * out.
+   */
+  if (info == &pkg->installed &&
+      (pkg->want != want_unknown ||
+       pkg->eflag != eflagv_ok ||
+       pkg->status != stat_notinstalled ||
+       informativeversion(&pkg->configversion)))
+    /* We ignore Section and Priority, as these tend to hang around. */
+    return 1;
   if (!info->valid) return 0;
   if (info->depends ||
       nes(info->description) ||
       nes(info->maintainer) ||
-      nes(info->version) ||
-      nes(info->revision) ||
+      nes(info->source) ||
+      nes(info->architecture) ||
+      informativeversion(&info->version) ||
       info->conffiles ||
       info->arbs) return 1;
   return 0;
 }
 
-int informative(struct pkginfo *pkg) {
-  return ((pkg->want != want_unknown && pkg->want != want_purge) ||
-          pkg->eflag != eflagv_ok ||
-          pkg->status != stat_notinstalled ||
-          nes(pkg->section) ||
-          pkg->priority != pri_unknown);
-}
-
 struct pkginfo *findpackage(const char *name) {
   struct pkginfo **pointerp, *newpkg;
 
