@@ -25,7 +25,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <sys/file.h>
 
 #include "config.h"
@@ -36,9 +36,14 @@ static char *dblockfile= 0;
 static int dblockfd= -1;
 
 static void cu_unlockdb(int argc, void **argv) {
+  struct flock fl;
   assert(dblockfile);
   assert(dblockfd >= 0);
-  if (flock(dblockfd,LOCK_UN))
+  fl.l_type= F_UNLCK;
+  fl.l_whence= SEEK_SET;
+  fl.l_start= 0;
+  fl.l_len= 1;
+  if (fcntl(dblockfd,F_SETLK,&fl) == -1)
     ohshite("unable to unlock dpkg status database");
 }
 
@@ -48,6 +53,7 @@ void unlockdatabase(const char *admindir) {
 
 void lockdatabase(const char *admindir) {
   int n;
+  struct flock fl;
   
   if (!dblockfile) {
     n= strlen(admindir);
@@ -63,7 +69,11 @@ void lockdatabase(const char *admindir) {
       ohshite("unable to open/create status database lockfile");
     }
   }
-  if (flock(dblockfd,LOCK_EX|LOCK_NB)) {
+  fl.l_type= F_WRLCK;
+  fl.l_whence= SEEK_SET;
+  fl.l_start= 0;
+  fl.l_len= 1;
+  if (fcntl(dblockfd,F_SETLK,&fl) == -1) {
     if (errno == EWOULDBLOCK || errno == EAGAIN)
       ohshit("status database area is locked - another dpkg/dselect is running");
     ohshite("unable to lock dpkg status database");
