@@ -88,23 +88,24 @@ const struct namevalue statusinfos[]= {  /* Note !  These must be in order ! */
 };
 
 const struct namevalue eflaginfos[]= {  /* Note !  These must be in order ! */
-  { "ok",             eflagv_ok         },
-  { "hold",           eflagv_hold       },
-  { "reinstreq",      eflagv_reinstreq  },
-  { "hold-reinstreq", eflagv_both       },
-  {  0                                  }
+  { "ok",                      eflagv_ok                 },
+  { "reinstreq",               eflagv_reinstreq          },
+  { "hold",                    eflagv_obsoletehold       },
+  { "hold-reinstreq",          eflagv_obsoleteboth       },
+  {  0                                                   }
 };
 
 const struct namevalue wantinfos[]= {  /* Note !  These must be in order ! */
   { "unknown",   want_unknown    },
   { "install",   want_install    },
+  { "hold",      want_hold       },
   { "deinstall", want_deinstall  },
   { "purge",     want_purge      },
   {  0                           }
 };
 
 const char *illegal_packagename(const char *p, const char **ep) {
-  static const char alsoallowed[]= "-_+.@:=%";
+  static const char alsoallowed[]= "-+._"; /* _ is deprecated */
   static char buf[150];
   int c;
   
@@ -125,11 +126,85 @@ const char *illegal_packagename(const char *p, const char **ep) {
 }
 
 const struct nickname nicknames[]= {
-  { "Package_revision",  "Revision"  },
-  { "Package-revision",  "Revision"  },
-  {  0                               }
+  /* NB: capitalisation of these strings is important. */
+  { "Recommended",       "Recommends"  },
+  { "Optional",          "Suggests"    },
+  { "Class",             "Priority"    },
+  { "Package-Revision",  "Revision"    },
+  { "Package_Revision",  "Revision"    },
+  {  0                                 }
 };
 
+int informativeversion(const struct versionrevision *version) {
+  return (version->epoch ||
+          (version->version && *version->version) ||
+          (version->revision && *version->revision));
+}
+
+void varbufversion(struct varbuf *vb,
+                   const struct versionrevision *version,
+                   enum versiondisplayepochwhen vdew) {
+  switch (vdew) {
+  case vdew_never:
+    break;
+  case vdew_nonambig:
+    if (!version->epoch &&
+        (!version->version || !strchr(version->version,':')) &&
+        (!version->revision || !strchr(version->revision,':'))) break;
+  case vdew_always: /* FALL THROUGH */
+    varbufprintf(vb,"%lu:",version->epoch);
+    break;
+  default:
+    internerr("bad versiondisplayepochwhen in varbufversion");
+  }
+  if (version->version) varbufaddstr(vb,version->version);
+  if (version->revision && *version->revision) {
+    varbufaddc(vb,'-');
+    varbufaddstr(vb,version->revision);
+  }
+}
+
+const char *versiondescribe(const struct versionrevision *version,
+                            enum versiondisplayepochwhen vdew) {
+  static struct varbuf bufs[10];
+  static int bufnum=0;
+
+  struct varbuf *vb;
+  
+  if (!informativeversion(version)) return "<none>";
+
+  vb= &bufs[bufnum]; bufnum++; if (bufnum == 10) bufnum= 0;
+  varbufreset(vb);
+  varbufversion(vb,version,vdew);
+  varbufaddc(vb,0);
+
+  return vb->buf;
+}
+
+const char *parseversion(struct versionrevision *rversion, const char *string) {
+  char *hyphen, *colon, *eepochcolon;
+  unsigned long epoch;
+
+  if (!*string) return "version string is empty";
+  
+  colon= strchr(string,':');
+  if (colon) {
+    epoch= strtoul(string,&eepochcolon,10);
+    if (colon != eepochcolon) return "epoch in version is not number";
+    if (!*++colon) return "nothing after colon in version number";
+    string= colon+1;
+    rversion->epoch= epoch;
+  } else {
+    rversion->epoch= 0;
+  }
+  rversion->version= nfstrsave(string);
+  hyphen= strrchr(rversion->version,'-');
+  if (hyphen) *hyphen++= 0;
+  rversion->revision= hyphen ? hyphen : nfstrsave("");
+  
+  return 0;
+}
+
 void parsemustfield(FILE *file, const char *filename, int lno,
                     FILE *warnto, int *warncount,
                     const struct pkginfo *pigp, int warnonly,
