@@ -27,8 +27,8 @@
 #include <assert.h>
 #include <time.h>
 #include <ctype.h>
+#include <fcntl.h>
 #include <sys/stat.h>
-#include <sys/fcntl.h>
 #include <sys/types.h>
 
 #include "config.h"
@@ -41,6 +41,70 @@
 #include "main.h"
 #include "archives.h"
 
+int filesavespackage(struct fileinlist *file, struct pkginfo *pkgtobesaved,
+                     struct pkginfo *pkgbeinginstalled) {
+  struct pkginfo *divpkg, *thirdpkg;
+  struct filepackages *packageslump;
+  int i;
+  
+  debug(dbg_eachfiledetail,"filesavespackage file `%s' package %s",
+        file->namenode->name,pkgtobesaved->name);
+  /* A package can only be saved by a file or directory which is part
+   * only of itself - it must be neither part of the new package being
+   * installed nor part of any 3rd package (this is important so that
+   * shared directories don't stop packages from disappearing).
+   */
+  /* Is the file in the package being installed ?  If so then it can't save.
+   */
+  if (file->namenode->flags & fnnf_new_inarchive) {
+    debug(dbg_eachfiledetail,"filesavespackage ... in new archive -- no save");
+    return 0;
+  }
+  /* If the file is a contended one and it's overridden by either
+   * the package we're considering disappearing or the package
+   * we're installing then they're not actually the same file, so
+   * we can't disappear the package - it is saved by this file.
+   */
+  if (file->namenode->divert && file->namenode->divert->useinstead) {
+    divpkg= file->namenode->divert->pkg;
+    if (divpkg == pkgtobesaved || divpkg == pkgbeinginstalled) {
+      debug(dbg_eachfiledetail,"filesavespackage ... diverted -- save!");
+      return 1;
+    }
+  }
+  /* Look for a 3rd package which can take over the file (in case
+   * it's a directory which is shared by many packages.
+   */
+  for (packageslump= file->namenode->packages;
+       packageslump;
+       packageslump= packageslump->more) {
+    for (i=0; i < PERFILEPACKAGESLUMP && packageslump->pkgs[i]; i++) {
+      thirdpkg= packageslump->pkgs[i];
+      debug(dbg_eachfiledetail, "filesavespackage ... also in %s",
+            thirdpkg->name);
+      /* Is this not the package being installed or the one being
+       * checked for disappearance ?
+       */
+      if (thirdpkg == pkgbeinginstalled || thirdpkg == pkgtobesaved) continue;
+      /* If !fileslistvalid then we've already disappeared this one, so
+       * we shouldn't try to make it take over this shared directory.
+       */
+      debug(dbg_eachfiledetail,"filesavespackage ...  is 3rd package");
+
+      if (!thirdpkg->clientdata->fileslistvalid) {
+        debug(dbg_eachfiledetail,
+              "process_archive ...  already disappeared !");
+        continue;
+      }
+      /* We've found a package that can take this file. */
+      debug(dbg_eachfiledetail, "filesavespackage ...  taken -- no save");
+      return 0;
+    }
+  }
+  debug(dbg_eachfiledetail, "filesavespackage ... not taken -- save !");
+  return 1;
+}
+
 void cu_pathname(int argc, void **argv) {
   ensure_pathname_nonexisting((char*)(argv[0]));
 } 
@@ -67,11 +131,31 @@ struct packageinlist *deconfigure= 0;
 
 static time_t currenttime;
 
-static void newtarobject_utime(struct TarInfo *ti) {
+static int does_replace(struct pkginfo *newpigp,
+                          struct pkginfoperfile *newpifp,
+                          struct pkginfo *oldpigp) {
+  struct dependency *dep;
+  
+  debug(dbg_depcon,"does_replace new=%s old=%s (%s)",newpigp->name,
+        oldpigp->name,versiondescribe(&oldpigp->installed.version,
+                                      vdew_always));
+  for (dep= newpifp->depends; dep; dep= dep->next) {
+    if (dep->type != dep_replaces || dep->list->ed != oldpigp) continue;
+    debug(dbg_depcondetail,"does_replace ... found old, version %s",
+          versiondescribe(&dep->list->version,vdew_always));
+    if (!versionsatisfied(&oldpigp->installed,dep->list)) continue;
+    debug(dbg_depcon,"does_replace ... yes");
+    return 1;
+  }
+  debug(dbg_depcon,"does_replace ... no");
+  return 0;
+}
+
+static void newtarobject_utime(const char *path, struct TarInfo *ti) {
   struct utimbuf utb;
   utb.actime= currenttime;
   utb.modtime= ti->ModTime;
-  if (utime(fnamenewvb.buf,&utb))
+  if (utime(path,&utb))
     ohshite("error setting timestamps of `%.255s'",ti->Name);
 }
 
@@ -80,7 +164,7 @@ static void newtarobject_allmodes(const char *path, struct TarInfo *ti) {
     ohshite("error setting ownership of `%.255s'",ti->Name);
   if (chmod(path,ti->Mode & ~S_IFMT))
     ohshite("error setting permissions of `%.255s'",ti->Name);
-  newtarobject_utime(ti);
+  newtarobject_utime(path,ti);
 }
 
 void setupfnamevbs(const char *filename) {
@@ -122,18 +206,20 @@ int unlinkorrmdir(const char *filename) {
         filename, r ? strerror(e) : "OK");
   errno= e; return r;
 }
-
+     
 int tarobject(struct TarInfo *ti) {
-  static struct varbuf conffderefn, hardlinkfn;
+  static struct varbuf conffderefn, hardlinkfn, symlinkfn;
   const char *usename;
     
   struct tarcontext *tc= (struct tarcontext*)ti->UserData;
-  int statr, fd, r;
+  int statr, fd, r, i, existingdirectory;
   struct stat stab, stabd;
   size_t sz, wsz;
   FILE *thefile;
   char databuf[TARBLKSZ];
   struct fileinlist *nifd;
+  struct pkginfo *divpkg, *otherpkg;
+  struct filepackages *packageslump;
 
   /* Append to list of files.
    * The trailing / put on the end of names in tarfiles has already
@@ -145,15 +231,30 @@ int tarobject(struct TarInfo *ti) {
   nifd->namenode->flags |= fnnf_new_inarchive;
 
   debug(dbg_eachfile,
-        "tarobject ti->Name=`%s' Mode=%lo owner=%u.%u Type=%d(%c) "
-        "          ti->LinkName=`%s'    namenode=`%s' flags=%o",
+        "tarobject ti->Name=`%s' Mode=%lo owner=%u.%u Type=%d(%c)"
+        " ti->LinkName=`%s' namenode=`%s' flags=%o instead=`%s'",
         ti->Name, (long)ti->Mode, (unsigned)ti->UserID, (unsigned)ti->GroupID, ti->Type,
         ti->Type == '\0' ? '_' :
         ti->Type >= '0' && ti->Type <= '6' ? "-hlcbdp"[ti->Type - '0'] : '?',
         ti->LinkName,
-        nifd->namenode->name, nifd->namenode->flags);
-        
-  usename= ti->Name;
+        nifd->namenode->name, nifd->namenode->flags,
+        nifd->namenode->divert && nifd->namenode->divert->useinstead
+        ? nifd->namenode->divert->useinstead->name : "<none>");
+
+  if (nifd->namenode->divert && nifd->namenode->divert->camefrom) {
+    divpkg= nifd->namenode->divert->pkg;
+    forcibleerr(fc_overwritediverted,
+                "trying to overwrite `%.250s', which is the "
+                "diverted version of `%.250s'%.10s%.100s%.10s",
+                nifd->namenode->name,
+                nifd->namenode->divert->camefrom->name,
+                divpkg ? " (package: " : "",
+                divpkg ? divpkg->name : "",
+                divpkg ? ")" : "");
+  }
+
+  usename= namenodetouse(nifd->namenode,tc->pkg)->name + 1; /* Skip the leading `/' */
+
   if (nifd->namenode->flags & fnnf_new_conff) {
     /* If it's a conffile we have to extract it next to the installed
      * version (ie, we do the usual link-following).
@@ -189,12 +290,82 @@ int tarobject(struct TarInfo *ti) {
   } else {
     debug(dbg_eachfiledetail,"tarobject already exists");
   }
+
+  /* Check to see if it's a directory or link to one and we don't need to
+   * do anything.  This has to be done now so that we don't die due to
+   * a file overwriting conflict.
+   */
+  existingdirectory= 0;
+  switch (ti->Type) {
+  case SymbolicLink:
+    /* If it's already an existing directory, do nothing. */
+    if (!statr && S_ISDIR(stab.st_mode)) {
+      debug(dbg_eachfiledetail,"tarobject SymbolicLink exists as directory");
+      existingdirectory= 1;
+    }
+    break;
+  case Directory:
+    /* If it's already an existing directory, do nothing. */
+    if (!stat(fnamevb.buf,&stabd) && S_ISDIR(stabd.st_mode)) {
+      debug(dbg_eachfiledetail,"tarobject Directory exists");
+      existingdirectory= 1;
+    }
+    break;
+  case NormalFile0: case NormalFile1:
+  case CharacterDevice: case BlockDevice:
+  case HardLink:
+    break;
+  default:
+    ohshit("archive contained object `%.255s' of unknown type 0x%x",ti->Name,ti->Type);
+  }
+
+  if (!existingdirectory) {
+    for (packageslump= nifd->namenode->packages;
+         packageslump;
+         packageslump= packageslump->more) {
+      for (i=0; i < PERFILEPACKAGESLUMP && packageslump->pkgs[i]; i++) {
+        otherpkg= packageslump->pkgs[i];
+        if (otherpkg == tc->pkg) continue;
+        debug(dbg_eachfile, "tarobject ... found in %s",otherpkg->name);
+        if (nifd->namenode->divert && nifd->namenode->divert->useinstead) {
+          /* Right, so we may be diverting this file.  This makes the conflict
+           * OK iff one of us is the diverting package (we don't need to
+           * check for both being the diverting package, obviously).
+           */
+          divpkg= nifd->namenode->divert->pkg;
+          debug(dbg_eachfile, "tarobject ... diverted, divpkg=%s",
+                divpkg ? divpkg->name : "<none>");
+          if (otherpkg == divpkg || tc->pkg == divpkg) continue;
+        }
+        /* Nope ?  Hmm, file conflict, perhaps.  Check Replaces. */
+        if (otherpkg->clientdata->replacingfilesandsaid) continue;
+        /* Is the package with the conflicting file in the `config files
+         * only' state ?  If so it must be a config file and we can
+         * silenty take it over.
+         */
+        if (otherpkg->status == stat_configfiles) continue;
+        /* Perhaps we're removing a conflicting package ? */
+        if (otherpkg->clientdata->istobe == itb_remove) continue;
+        if (does_replace(tc->pkg,&tc->pkg->available,otherpkg)) {
+          printf("Replacing files in old package %s ...\n",otherpkg->name);
+          otherpkg->clientdata->replacingfilesandsaid= 1;
+        } else {
+          forcibleerr(fc_overwrite,
+                      "trying to overwrite `%.250s', which is also in package %.250s",
+                      nifd->namenode->name,otherpkg->name);
+        }
+      }
+    }
+  }
+       
   /* Now, at this stage we want to make sure neither of .dpkg-new and .dpkg-tmp
    * are hanging around.
    */
   ensure_pathname_nonexisting(fnamenewvb.buf);
   ensure_pathname_nonexisting(fnametmpvb.buf);
 
+  if (existingdirectory) return 0;
+
   /* Now we start to do things that we need to be able to undo
    * if something goes wrong.
    */
@@ -214,11 +385,14 @@ int tarobject(struct TarInfo *ti) {
     for (sz= ti->Size; sz > 0; sz -= wsz) {
       wsz= sz > TARBLKSZ ? TARBLKSZ : sz;
       r= fread(databuf,1,TARBLKSZ,tc->backendpipe);
-      if (r<TARBLKSZ)
-        if (ferror(tc->backendpipe))
+      if (r<TARBLKSZ) {
+        if (ferror(tc->backendpipe)) {
           ohshite("error reading " BACKEND " during `%.255s'",ti->Name);
-        else
-          ohshit("unexpected EOF from " BACKEND " in `%.255s'",ti->Name);
+        } else {
+          errno= 0;
+          return -1;
+        }
+      }
       if (fwrite(databuf,1,wsz,thefile) != wsz)
         ohshite("error writing to `%.255s'",ti->Name);
     }
@@ -229,7 +403,7 @@ int tarobject(struct TarInfo *ti) {
     pop_cleanup(ehflag_normaltidy); /* thefile= fdopen(fd) */
     if (fclose(thefile))
       ohshite("error closing/writing `%.255s'",ti->Name);
-    newtarobject_utime(ti);
+    newtarobject_utime(fnamenewvb.buf,ti);
     break;
   case CharacterDevice: case BlockDevice:
     if (mknod(fnamenewvb.buf,ti->Mode & S_IFMT,ti->Device))
@@ -247,11 +421,7 @@ int tarobject(struct TarInfo *ti) {
     newtarobject_allmodes(fnamenewvb.buf,ti);
     break;
   case SymbolicLink:
-    /* If it's already an existing directory, do nothing. */
-    if (!statr && S_ISDIR(stab.st_mode)) {
-      debug(dbg_eachfiledetail,"tarobject SymbolicLink exists as directory");
-      return 0;
-    }
+    /* We've already cheched for an existing directory. */
     if (symlink(ti->LinkName,fnamenewvb.buf))
       ohshite("error creating symbolic link `%.255s'",ti->Name);
     debug(dbg_eachfiledetail,"tarobject SymbolicLink creating");
@@ -259,11 +429,7 @@ int tarobject(struct TarInfo *ti) {
       ohshite("error setting ownership of symlink `%.255s'",ti->Name);
     break;
   case Directory:
-    /* If it's already an existing directory, do nothing. */
-    if (!stat(fnamevb.buf,&stabd) && S_ISDIR(stabd.st_mode)) {
-      debug(dbg_eachfiledetail,"tarobject Directory exists");
-      return 0;
-    }
+    /* We've already checked for an existing directory. */
     if (mkdir(fnamenewvb.buf,
               ti->Mode & (S_IRUSR|S_IRGRP|S_IROTH | S_IXUSR|S_IXGRP|S_IXOTH)))
       ohshite("error creating directory `%.255s'",ti->Name);
@@ -271,7 +437,7 @@ int tarobject(struct TarInfo *ti) {
     newtarobject_allmodes(fnamenewvb.buf,ti);
     break;
   default:
-    ohshit("archive contained object `%.255s' of unknown type 0x%x",ti->Name,ti->Type);
+    internerr("bad tar type, but already checked");
   }
   /*
    * Now we have extracted the new object in .dpkg-new (or, if the
@@ -285,6 +451,7 @@ int tarobject(struct TarInfo *ti) {
    */
   if (nifd->namenode->flags & fnnf_new_conff) {
     debug(dbg_conffdetail,"tarobject conffile extracted");
+    nifd->namenode->flags |= fnnf_elide_other_lists;
     return 0;
   }
 
@@ -295,13 +462,27 @@ int tarobject(struct TarInfo *ti) {
   if (statr) { /* Don't try to back it up if it didn't exist. */
     debug(dbg_eachfiledetail,"tarobject new - no backup");
   } else {
-    if (ti->Type == Directory || ti->Type == SymbolicLink ||
-        S_ISDIR(stab.st_mode) || S_ISLNK(stab.st_mode)) {
+    if (ti->Type == Directory || S_ISDIR(stab.st_mode)) {
+      /* One of the two is a directory - can't do atomic install. */
       debug(dbg_eachfiledetail,"tarobject directory, nonatomic");
-      /* One of the two is a directory or a symlink - can't do atomic install. */
       nifd->namenode->flags |= fnnf_no_atomic_overwrite;
       if (rename(fnamevb.buf,fnametmpvb.buf))
         ohshite("unable to move aside `%.255s' to install new version",ti->Name);
+    } else if (S_ISLNK(stab.st_mode)) {
+      /* We can't make a symlink with two hardlinks, so we'll have to copy it.
+       * (Pretend that making a copy of a symlink is the same as linking to it.)
+       */
+      varbufreset(&symlinkfn);
+      do {
+        varbufextend(&symlinkfn);
+        r= readlink(fnamevb.buf,symlinkfn.buf,symlinkfn.size);
+        if (r<0) ohshite("unable to read link `%.255s'",ti->Name);
+      } while (r == symlinkfn.size);
+      symlinkfn.used= r; varbufaddc(&symlinkfn,0);
+      if (symlink(symlinkfn.buf,fnametmpvb.buf))
+        ohshite("unable to make backup symlink for `%.255s'",ti->Name);
+      if (chown(fnametmpvb.buf,stab.st_uid,stab.st_gid))
+        ohshite("unable to chown backup symlink for `%.255s'",ti->Name);
     } else {
       debug(dbg_eachfiledetail,"tarobject nondirectory, `link' backup");
       if (link(fnamevb.buf,fnametmpvb.buf))
@@ -330,6 +511,18 @@ static int try_remove_can(struct deppossi *pdep,
             fixbyrm->name, why);
     return 1;
   } else if (f_autodeconf) {
+    if (pdep->up->up->installed.essential) {
+      if (fc_removeessential) {
+        fprintf(stderr, DPKG ": warning - considering deconfiguration of essential\n"
+                " package %s, to enable removal of %s.\n",
+                pdep->up->up->name,fixbyrm->name);
+      } else {
+        fprintf(stderr, DPKG ": no, %s is essential, will not deconfigure\n"
+                " it in order to enable removal of %s.\n",
+                pdep->up->up->name,fixbyrm->name);
+        return 0;
+      }
+    }
     pdep->up->up->clientdata->istobe= itb_deconfigure;
     newdeconf= m_malloc(sizeof(struct packageinlist));
     newdeconf->next= deconfigure;
@@ -337,7 +530,7 @@ static int try_remove_can(struct deppossi *pdep,
     deconfigure= newdeconf;
     return 1;
   } else {
-    fprintf(stderr, DPKG ": no, cannot remove %s:\n%s",
+    fprintf(stderr, DPKG ": no, cannot remove %s (--auto-deconfigure will help):\n%s",
             fixbyrm->name, why);
     return 0;
   }
@@ -354,79 +547,87 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
   varbufinit(&removalwhy);
 
   fixbyrm= 0;
-  if (depisok(dep, &conflictwhy, *conflictorp ? 0 : &fixbyrm)) {
+  if (depisok(dep, &conflictwhy, *conflictorp ? 0 : &fixbyrm, 0)) {
     varbuffree(&conflictwhy);
     varbuffree(&removalwhy);
     return;
   }
-  if (fixbyrm &&
-      ((pkg->available.essential && fixbyrm->installed.essential) ||
-       (fixbyrm->want != want_install &&
-        (!fixbyrm->installed.essential || fc_removeessential)))) {
+  if (fixbyrm) {
     ensure_package_clientdata(fixbyrm);
-    assert(fixbyrm->clientdata->istobe == itb_normal);
-    fixbyrm->clientdata->istobe= itb_remove;
-    fprintf(stderr, DPKG ": considering removing %s in favour of %s ...\n",
-            fixbyrm->name, pkg->name);
-    if (fixbyrm->status != stat_installed) {
-      fprintf(stderr,
-              "%s is not properly installed - ignoring any dependencies on it.\n",
-              fixbyrm->name);
-      pdep= 0;
-    } else {
-      for (pdep= fixbyrm->installed.depended;
-           pdep;
-           pdep= pdep->nextrev) {
-        if (pdep->up->type != dep_depends) continue;
-        if (depisok(pdep->up, &removalwhy, 0)) continue;
-        varbufaddc(&removalwhy,0);
-        if (!try_remove_can(pdep,fixbyrm,removalwhy.buf))
-          break;
-      }
-      if (!pdep) {
-        /* If we haven't found a reason not to yet, let's look some more. */
-        for (providecheck= fixbyrm->installed.depends;
-             providecheck;
-             providecheck= providecheck->next) {
-          if (providecheck->type != dep_provides) continue;
-          for (pdep= providecheck->list->ed->installed.depended;
-               pdep;
-               pdep= pdep->nextrev) {
-            if (pdep->up->type != dep_depends) continue;
-            if (depisok(pdep->up, &removalwhy, 0)) continue;
-            varbufaddc(&removalwhy,0);
-            fprintf(stderr, DPKG
-                    ": may have trouble removing %s, as it provides %s ...\n",
-                    fixbyrm->name, providecheck->list->ed->name);
-            if (!try_remove_can(pdep,fixbyrm,removalwhy.buf))
-              goto break_from_both_loops_at_once;
-          }
-        }
-      break_from_both_loops_at_once:;
-      }
+    if (fixbyrm->clientdata->istobe == itb_installnew) {
+      fixbyrm= dep->up;
+      ensure_package_clientdata(fixbyrm);
     }
-    if (!pdep && skip_due_to_hold(fixbyrm)) {
-      pdep= &flagdeppossi;
-    }
-    if (!pdep && (fixbyrm->eflag & eflagf_reinstreq)) {
-      if (fc_removereinstreq) {
-        fprintf(stderr, DPKG ": package %s requires reinstallation, but will"
-                " remove anyway as you request.\n", fixbyrm->name);
-      } else {
-        fprintf(stderr, DPKG ": package %s requires reinstallation, will not remove.\n",
+    if (((pkg->available.essential && fixbyrm->installed.essential) ||
+         (((fixbyrm->want != want_install && fixbyrm->want != want_hold) ||
+           does_replace(pkg,&pkg->available,fixbyrm)) &&
+          (!fixbyrm->installed.essential || fc_removeessential)))) {
+      assert(fixbyrm->clientdata->istobe == itb_normal);
+      fixbyrm->clientdata->istobe= itb_remove;
+      fprintf(stderr, DPKG ": considering removing %s in favour of %s ...\n",
+              fixbyrm->name, pkg->name);
+      if (fixbyrm->status != stat_installed) {
+        fprintf(stderr,
+                "%s is not properly installed - ignoring any dependencies on it.\n",
                 fixbyrm->name);
+        pdep= 0;
+      } else {
+        for (pdep= fixbyrm->installed.depended;
+             pdep;
+             pdep= pdep->nextrev) {
+          if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends)
+            continue;
+          if (depisok(pdep->up, &removalwhy, 0,0)) continue;
+          varbufaddc(&removalwhy,0);
+          if (!try_remove_can(pdep,fixbyrm,removalwhy.buf))
+            break;
+        }
+        if (!pdep) {
+          /* If we haven't found a reason not to yet, let's look some more. */
+          for (providecheck= fixbyrm->installed.depends;
+               providecheck;
+               providecheck= providecheck->next) {
+            if (providecheck->type != dep_provides) continue;
+            for (pdep= providecheck->list->ed->installed.depended;
+                 pdep;
+                 pdep= pdep->nextrev) {
+              if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends)
+                continue;
+              if (depisok(pdep->up, &removalwhy, 0,0)) continue;
+              varbufaddc(&removalwhy,0);
+              fprintf(stderr, DPKG
+                      ": may have trouble removing %s, as it provides %s ...\n",
+                      fixbyrm->name, providecheck->list->ed->name);
+              if (!try_remove_can(pdep,fixbyrm,removalwhy.buf))
+                goto break_from_both_loops_at_once;
+            }
+          }
+        break_from_both_loops_at_once:;
+        }
+      }
+      if (!pdep && skip_due_to_hold(fixbyrm)) {
         pdep= &flagdeppossi;
       }
+      if (!pdep && (fixbyrm->eflag & eflagf_reinstreq)) {
+        if (fc_removereinstreq) {
+          fprintf(stderr, DPKG ": package %s requires reinstallation, but will"
+                  " remove anyway as you request.\n", fixbyrm->name);
+        } else {
+          fprintf(stderr, DPKG ": package %s requires reinstallation, "
+                  "will not remove.\n", fixbyrm->name);
+          pdep= &flagdeppossi;
+        }
+      }
+      if (!pdep) {
+        /* This conflict is OK - we'll remove the conflictor. */
+        *conflictorp= fixbyrm;
+        varbuffree(&conflictwhy); varbuffree(&removalwhy);
+        fprintf(stderr, DPKG ": yes, will remove %s in favour of %s.\n",
+                fixbyrm->name, pkg->name);
+        return;
+      }
+      fixbyrm->clientdata->istobe= itb_normal; /* put it back */
     }
-    if (!pdep) {
-      /* This conflict is OK - we'll remove the conflictor. */
-      *conflictorp= fixbyrm;
-      varbuffree(&conflictwhy); varbuffree(&removalwhy);
-      fprintf(stderr, DPKG ": yes, will remove %s in favour of %s.\n",
-              fixbyrm->name, pkg->name);
-      return;
-    }
-    fixbyrm->clientdata->istobe= itb_normal; /* put it back */
   }
   varbufaddc(&conflictwhy,0);
   fprintf(stderr, DPKG ": regarding %s containing %s:\n%s",
@@ -465,6 +666,8 @@ void archivefiles(const char *const *argv) {
   const char **arglist;
   char *p;
 
+  checkpath();
+  
   if (f_recursive) {
     
     if (!*argv)
@@ -476,7 +679,7 @@ void archivefiles(const char *const *argv) {
       int i;
       m_dup2(pi[1],1); close(pi[0]);
       for (i=0, ap=argv; *ap; ap++, i++);
-      arglist= m_malloc(sizeof(char*)*(i+10));
+      arglist= m_malloc(sizeof(char*)*(i+15));
       arglist[0]= FIND;
       for (i=1, ap=argv; *ap; ap++, i++) {
         if (strchr(FIND_EXPRSTARTCHARS,(*ap)[0])) {
@@ -489,7 +692,8 @@ void archivefiles(const char *const *argv) {
           arglist[i]= *ap;
         }
       }
-      arglist[i++]= "-name";
+      arglist[i++]= "-follow"; /*  When editing these, make sure that     */
+      arglist[i++]= "-name";   /*  arglist is mallocd big enough, above.  */
       arglist[i++]= ARCHIVE_FILENAME_PATTERN;
       arglist[i++]= "-type";
       arglist[i++]= "f";
@@ -545,6 +749,8 @@ void archivefiles(const char *const *argv) {
   varbufaddstr(&fnametmpvb,instdir); varbufaddc(&fnametmpvb,'/');
   varbufaddstr(&fnamenewvb,instdir); varbufaddc(&fnamenewvb,'/');
   fnameidlu= fnamevb.used;
+
+  ensure_diversions();
   
   while ((thisarg= *argp++) != 0) {
     if (setjmp(ejbuf)) {
