@@ -23,9 +23,9 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/fcntl.h>
 #include <dirent.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -124,6 +124,14 @@ void deferred_configure(struct pkginfo *pkg) {
   if (pkg->status == stat_unpacked) {
     debug(dbg_general,"deferred_configure updating conffiles");
 
+    /* This will not do at all the right thing with overridden conffiles
+     * or conffiles that are the `target' of an override; all the references
+     * here would be to the `contested' filename, and in any case there'd
+     * only be one hash for both `versions' of the conffile.
+     *
+     * Overriding conffiles is a silly thing to do anyway :-).
+     */
+
     modstatdb_note(pkg);
 
     /* On entry, the `new' version of each conffile has been
@@ -169,8 +177,13 @@ void deferred_configure(struct pkginfo *pkg) {
         if (errno != ENOENT)
           ohshite("unable to stat current installed conffile `%.250s'",cdr.buf);
       }
-      
-      if (!strcmp(conff->hash,NEWCONFFILEFLAG)) {
+
+      if (!strcmp(currenthash,newdisthash)) {
+        /* They're both the same so there's no point asking silly questions. */
+        useredited= -1;
+        distedited= -1;
+        what= cfo_identical;
+      } else if (!strcmp(conff->hash,NEWCONFFILEFLAG)) {
         if (!strcmp(currenthash,NONEXISTENTFLAG)) {
           what= cfo_newconff;
           useredited= -1;
@@ -364,7 +377,12 @@ void deferred_configure(struct pkginfo *pkg) {
   modstatdb_note(pkg);
 
   if (maintainer_script_installed(pkg, POSTINSTFILE, "post-installation",
-                                  "configure", (char*)0))
+                                  "configure",
+                                  informativeversion(&pkg->configversion)
+                                  ? versiondescribe(&pkg->configversion,
+                                                  vdew_nonambig)
+                                  : "",
+                                  (char*)0))
     putchar('\n');
 
   pkg->status= stat_installed;
