@@ -28,8 +28,8 @@
 #include <time.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <fcntl.h>
 #include <sys/stat.h>
-#include <sys/fcntl.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -59,14 +59,14 @@ void process_archive(const char *filename) {
   static char *cidirbuf=0, *reasmbuf=0;
   static struct fileinlist *newconffiles;
   static enum pkgstatus oldversionstatus;
-  static struct varbuf infofnvb, fnvb, removalwhy;
+  static struct varbuf infofnvb, fnvb, depprobwhy;
   static struct tarcontext tc;
   
   int c1, r, admindirlen, i, infodirlen, infodirbaseused, status;
   struct pkgiterator *it;
-  struct pkginfo *pkg, *conflictor, *otherpkg;
+  struct pkginfo *pkg, *conflictor, *otherpkg, *divpkg;
   char *cidir, *cidirrest, *p;
-  char pfilenamebuf[50], conffilenamebuf[MAXCONFFILENAME];
+  char pfilenamebuf[35], conffilenamebuf[MAXCONFFILENAME];
   const char *pfilename, *newinfofilename;
   struct fileinlist *newconff, **newconffileslastp, *newfileslist;
   struct fileinlist *cfile;
@@ -82,7 +82,8 @@ void process_archive(const char *filename) {
   struct dirent *de;
   struct stat stab;
   struct packageinlist *deconpil, *deconpiltemp;
-
+  enum versiondisplayepochwhen needepochs;
+  
   cleanup_pkg_failed= cleanup_conflictor_failed= 0;
   admindirlen= strlen(admindir);
 
@@ -168,13 +169,18 @@ void process_archive(const char *filename) {
           &pkg,0,0);
 
   if (cipaction->arg == act_avail) {
-    printf("Read updated information about %s from %s.\n",pkg->name,pfilename);
+    printf("Recorded info about %s from %s.\n",pkg->name,pfilename);
     pop_cleanup(ehflag_normaltidy);
     return;
   }
 
-  if (skip_due_to_hold(pkg)) { pop_cleanup(ehflag_normaltidy); return; }
-  
+  if (pkg->available.architecture && *pkg->available.architecture &&
+      strcmp(pkg->available.architecture,"all") &&
+      strcmp(pkg->available.architecture,architecture))
+    forcibleerr(fc_architecture,
+                "package architecture (%s) does not match system (%s)",
+                pkg->available.architecture,architecture);
+    
   if (!pkg->installed.valid) blankpackageperfile(&pkg->installed);
   assert(pkg->available.valid);
 
@@ -198,27 +204,28 @@ void process_archive(const char *filename) {
   }
 
   if (pkg->status == stat_installed) {
-    r= versioncompare(pkg->available.version,pkg->available.revision,
-                      pkg->installed.version,pkg->installed.revision);
+    r= versioncompare(&pkg->available.version,&pkg->installed.version);
     if (r < 0) {
+      needepochs= epochsdiffer(&pkg->available.version,&pkg->installed.version) ?
+        vdew_always : vdew_never;
       if (fc_downgrade) {
         fprintf(stderr, DPKG " - warning: downgrading %.250s from %.250s to %.250s.\n",
                 pkg->name,
-                versiondescribe(pkg->installed.version,pkg->installed.revision),
-                versiondescribe(pkg->available.version,pkg->available.revision));
+                versiondescribe(&pkg->installed.version,needepochs),
+                versiondescribe(&pkg->available.version,needepochs));
       } else {
         fprintf(stderr, "Will not downgrade"
                 " %.250s from version %.250s to %.250s, skipping.\n",
                 pkg->name,
-                versiondescribe(pkg->installed.version,pkg->installed.revision),
-                versiondescribe(pkg->available.version,pkg->available.revision));
+                versiondescribe(&pkg->installed.version,needepochs),
+                versiondescribe(&pkg->available.version,needepochs));
         pop_cleanup(ehflag_normaltidy);
         return;
       }
     } else if (r == 0 && f_skipsame && /* same version fully installed ? */
                pkg->status == stat_installed && !(pkg->eflag &= eflagf_reinstreq)) {
       fprintf(stderr, "Version %.250s of %.250s already installed, skipping.\n",
-              versiondescribe(pkg->installed.version,pkg->installed.revision),
+              versiondescribe(&pkg->installed.version,vdew_never),
               pkg->name);
       pop_cleanup(ehflag_normaltidy);
       return;
@@ -244,9 +251,18 @@ void process_archive(const char *filename) {
         }
       }
       break;
-    case dep_suggests: case dep_recommends: case dep_depends:
+    case dep_suggests: case dep_recommends: case dep_depends: case dep_replaces:
       /* Ignore these here. */
       break;
+    case dep_predepends:
+      if (!depisok(dsearch,&depprobwhy,0,1)) {
+        varbufaddc(&depprobwhy,0);
+        fprintf(stderr, DPKG ": regarding %s containing %s, pre-dependency problem:\n%s",
+                pfilename, pkg->name, depprobwhy.buf);
+        if (!force_depends(dsearch->list))
+          ohshit("pre-dependency problem - not installing %.250s",pkg->name);
+        fprintf(stderr, DPKG ": warning - ignoring pre-dependency problem !\n");
+      }
     }
   }
   /* Look for things that conflict with us. */
@@ -256,7 +272,7 @@ void process_archive(const char *filename) {
   }
   
   ensure_allinstfiles_available();
-  zero_filesdb_info();
+  filesdbinit();
   
   if (pkg->status != stat_notinstalled && pkg->status != stat_configfiles)
     printf("Preparing to replace %s (using %s) ...\n",pkg->name,pfilename);
@@ -292,11 +308,46 @@ void process_archive(const char *filename) {
       *newconffileslastp= newconff;
       newconffileslastp= &newconff->next;
       newconff->namenode->oldhash= NEWCONFFILEFLAG;
-      if (pkg->status != stat_notinstalled) {
-        for (searchconff= pkg->installed.conffiles;
-             searchconff && strcmp(newconff->namenode->name,searchconff->name);
-             searchconff= searchconff->next);
-        if (searchconff) newconff->namenode->oldhash= searchconff->hash;
+      /* Let's see if any packages have this file.  If they do we
+       * check to see if they listed it as a conffile, and if they did
+       * we copy the hash across.  Since (for plain file conffiles,
+       * which is the only kind we are supposed to have) there will
+       * only be one package which `has' the file, this will usually
+       * mean we only look in the package which we're installing now.
+       * The `conffiles' data in the status file is ignored when a
+       * package isn't also listed in the file ownership database as
+       * having that file.  If several packages are listed as owning
+       * the file we pick one at random.
+       */
+      searchconff= 0;
+      for (packageslump= newconff->namenode->packages;
+           packageslump;
+           packageslump= packageslump->more) {
+        for (i=0; i < PERFILEPACKAGESLUMP && packageslump->pkgs[i]; i++) {
+          otherpkg= packageslump->pkgs[i];
+          debug(dbg_conffdetail,"process_archive conffile `%s' in package %s - conff ?",
+                newconff->namenode->name,otherpkg->name);
+          for (searchconff= otherpkg->installed.conffiles;
+               searchconff && strcmp(newconff->namenode->name,searchconff->name);
+               searchconff= searchconff->next)
+            debug(dbg_conffdetail,
+                  "process_archive conffile `%s' in package %s - conff ? not `%s'",
+                  newconff->namenode->name,otherpkg->name,searchconff->name);
+          if (searchconff) {
+            debug(dbg_conff,"process_archive conffile `%s' package=%s %s hash=%s",
+                  newconff->namenode->name,otherpkg->name,
+                  otherpkg == pkg ? "same" : "different!",
+                  searchconff->hash);
+            if (otherpkg == pkg) goto xit_conff_hashcopy_srch;
+          }
+        }
+      }
+    xit_conff_hashcopy_srch:
+      if (searchconff) {
+        newconff->namenode->oldhash= searchconff->hash;
+      } else {
+        debug(dbg_conff,"process_archive conffile `%s' no package, no hash",
+              newconff->namenode->name);
       }
       newconff->namenode->flags |= fnnf_new_conff;
     }
@@ -349,11 +400,11 @@ void process_archive(const char *filename) {
                    3,(void*)deconpil->pkg,(void*)conflictor,(void*)pkg);
       maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
                                   "deconfigure", "in-favour", pkg->name,
-                                  versiondescribe(pkg->available.version,
-                                                  pkg->available.revision),
+                                  versiondescribe(&pkg->available.version,
+                                                  vdew_nonambig),
                                   "removing", conflictor->name,
-                                  versiondescribe(conflictor->installed.version,
-                                                  conflictor->installed.revision),
+                                  versiondescribe(&conflictor->installed.version,
+                                                  vdew_nonambig),
                                   (char*)0);
     }
     conflictor->status= stat_halfconfigured;
@@ -362,14 +413,16 @@ void process_archive(const char *filename) {
                  2,(void*)conflictor,(void*)pkg);
     maintainer_script_installed(conflictor, PRERMFILE, "pre-removal",
                                 "remove", "in-favour", pkg->name,
-                                versiondescribe(pkg->available.version,
-                                                pkg->available.revision),
+                                versiondescribe(&pkg->available.version,
+                                                vdew_nonambig),
                                 (char*)0);
     conflictor->status= stat_halfinstalled;
     modstatdb_note(conflictor);
   }
 
   pkg->eflag |= eflagf_reinstreq;
+  if (pkg->status == stat_notinstalled)
+    pkg->installed.version= pkg->available.version;
   pkg->status= stat_halfinstalled;
   modstatdb_note(pkg);
   if (oldversionstatus == stat_notinstalled) {
@@ -381,15 +434,15 @@ void process_archive(const char *filename) {
     push_cleanup(cu_preinstnew,~ehflag_normaltidy, 0,0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
     maintainer_script_new(PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "install", versiondescribe(pkg->installed.version,
-                                                     pkg->installed.revision),
+                          "install", versiondescribe(&pkg->installed.version,
+                                                     vdew_nonambig),
                           (char*)0);
   } else {
     push_cleanup(cu_preinstupgrade,~ehflag_normaltidy, 0,0,
                  4,(void*)pkg,(void*)cidir,(void*)cidirrest,(void*)&oldversionstatus);
     maintainer_script_new(PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "upgrade", versiondescribe(pkg->installed.version,
-                                                     pkg->installed.revision),
+                          "upgrade", versiondescribe(&pkg->installed.version,
+                                                     vdew_nonambig),
                           (char*)0);
     printf("Unpacking replacement %.250s ...\n",pkg->name);
   }
@@ -480,12 +533,14 @@ void process_archive(const char *filename) {
 
   r= TarExtractor((void*)&tc, &tf);
   if (r) {
-    if (ferror(tc.backendpipe))
-      ohshite("error reading tar header from " BACKEND);
-    else if (feof(tc.backendpipe))
-      ohshit("unexpected EOF in tar header from " BACKEND);
-    else
-      ohshite("unpick of " BACKEND " tar output failed (%d) for unknown reason",r);
+    if (errno) {
+      ohshite("error reading " BACKEND " tar output");
+    } else if (feof(tc.backendpipe)) {
+      waitsubproc(c1,BACKEND " --fsys-tarfile (EOF)",1);
+      ohshit("unexpected EOF in filesystem tarfile - corrupted package archive");
+    } else {
+      ohshit("corrupted filesystem tarfile - corrupted package archive");
+    }
   }
   tmpf= tc.backendpipe;
   tc.backendpipe= 0;
@@ -532,7 +587,7 @@ void process_archive(const char *filename) {
       continue;
     if (isdirectoryinuse(namenode,pkg)) continue;
     fnamevb.used= fnameidlu;
-    varbufaddstr(&fnamevb,namenode->name);
+    varbufaddstr(&fnamevb, namenodetouse(namenode,pkg)->name);
     varbufaddc(&fnamevb,0);
     if (!rmdir(fnamevb.buf)) continue;
     if (errno == ENOENT || errno == ELOOP) continue;
@@ -662,10 +717,7 @@ void process_archive(const char *filename) {
       newpossi->ed= possi->ed;
       newpossi->next= 0; newpossi->nextrev= newpossi->backrev= 0;
       newpossi->verrel= possi->verrel;
-      if (possi->verrel != dvr_none) {
-        newpossi->version= possi->version;
-        newpossi->revision= possi->revision;
-      }
+      if (possi->verrel != dvr_none) newpossi->version= possi->version;
       newpossi->cyclebreak= 0;
       *newpossilastp= newpossi;
       newpossilastp= &newpossi->next;
@@ -690,8 +742,9 @@ void process_archive(const char *filename) {
   pkg->installed.essential= pkg->available.essential;
   pkg->installed.description= pkg->available.description;
   pkg->installed.maintainer= pkg->available.maintainer;
+  pkg->installed.source= pkg->available.source;
+  pkg->installed.architecture= 0; /* This is irrelevant in the status file. */
   pkg->installed.version= pkg->available.version;
-  pkg->installed.revision= pkg->available.revision;
 
   /* We have to generate our own conffiles structure. */
   pkg->installed.conffiles= 0; iconffileslastp= &pkg->installed.conffiles;
@@ -738,12 +791,9 @@ void process_archive(const char *filename) {
       continue;
     }
     for (cfile= otherpkg->clientdata->files;
-         cfile && (cfile->namenode->flags & fnnf_new_inarchive);
+         cfile && !filesavespackage(cfile,otherpkg,pkg);
          cfile= cfile->next);
-    if (cfile) {
-      debug(dbg_stupidlyverbose, "process_archive saved by `%s'",cfile->namenode->name);
-      continue;
-    }
+    if (cfile) continue;
 
     /* So dependency things will give right answers ... */
     otherpkg->clientdata->istobe= itb_remove; 
@@ -751,10 +801,11 @@ void process_archive(const char *filename) {
     for (pdep= otherpkg->installed.depended;
          pdep;
          pdep= pdep->nextrev) {
-      if (pdep->up->type != dep_depends && pdep->up->type != dep_recommends) continue;
-      if (depisok(pdep->up, &removalwhy, 0)) continue;
-      varbufaddc(&removalwhy,0);
-      debug(dbg_veryverbose,"process_archive cannot disappear: %s",removalwhy.buf);
+      if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends &&
+          pdep->up->type != dep_recommends) continue;
+      if (depisok(pdep->up, &depprobwhy, 0,0)) continue;
+      varbufaddc(&depprobwhy,0);
+      debug(dbg_veryverbose,"process_archive cannot disappear: %s",depprobwhy.buf);
       break;
     }
     if (!pdep) {
@@ -766,12 +817,13 @@ void process_archive(const char *filename) {
         for (pdep= providecheck->list->ed->installed.depended;
              pdep;
              pdep= pdep->nextrev) {
-          if (pdep->up->type != dep_depends && pdep->up->type != dep_recommends)
+          if (pdep->up->type != dep_depends && pdep->up->type != dep_predepends &&
+              pdep->up->type != dep_recommends)
             continue;
-          if (depisok(pdep->up, &removalwhy, 0)) continue;
-          varbufaddc(&removalwhy,0);
+          if (depisok(pdep->up, &depprobwhy, 0,0)) continue;
+          varbufaddc(&depprobwhy,0);
           debug(dbg_veryverbose,"process_archive cannot disappear (provides %s): %s",
-                providecheck->list->ed->name, removalwhy.buf);
+                providecheck->list->ed->name, depprobwhy.buf);
           goto break_from_both_loops_at_once;
         }
       }
@@ -790,8 +842,8 @@ void process_archive(const char *filename) {
     maintainer_script_installed(otherpkg, POSTRMFILE,
                                 "post-removal script (for disappearance)",
                                 "disappear", pkg->name, 
-                                versiondescribe(pkg->available.version,
-                                                pkg->available.revision),
+                                versiondescribe(&pkg->available.version,
+                                                vdew_nonambig),
                                 (char*)0);
 
     /* OK, now we delete all the stuff in the `info' directory .. */
@@ -827,8 +879,9 @@ void process_archive(const char *filename) {
 
     otherpkg->installed.depends= 0;
     otherpkg->installed.essential= 0;
-    otherpkg->installed.description= otherpkg->installed.maintainer=
-      otherpkg->installed.version= otherpkg->installed.revision= 0;
+    otherpkg->installed.description= otherpkg->installed.maintainer= 0;
+    otherpkg->installed.conffiles= 0;
+    blankversion(&otherpkg->installed.version);
     otherpkg->installed.arbs= 0;
     otherpkg->clientdata->fileslistvalid= 0;
 
@@ -847,8 +900,23 @@ void process_archive(const char *filename) {
    */
   for (cfile= newfileslist; cfile; cfile= cfile->next) {
     if (!(cfile->namenode->flags & fnnf_elide_other_lists)) continue;
-    debug(dbg_eachfile, "process_archive looking for overwriting `%s'",
-          cfile->namenode->name);
+    if (cfile->namenode->divert && cfile->namenode->divert->useinstead) {
+      divpkg= cfile->namenode->divert->pkg;
+      if (divpkg == pkg) {
+        debug(dbg_eachfile,
+              "process_archive not overwriting any `%s' (overriding, `%s')",
+              cfile->namenode->name, cfile->namenode->divert->useinstead->name);
+        continue;
+      } else {
+        debug(dbg_eachfile,
+              "process_archive looking for overwriting `%s' (overridden by %s)",
+              cfile->namenode->name, divpkg ? divpkg->name : "<local>");
+      }        
+    } else {
+      divpkg= 0;
+      debug(dbg_eachfile, "process_archive looking for overwriting `%s'",
+            cfile->namenode->name);
+    }
     for (packageslump= cfile->namenode->packages;
          packageslump;
          packageslump= packageslump->more) {
@@ -859,6 +927,10 @@ void process_archive(const char *filename) {
          * and we don't bother with it here, clearly.
          */
         if (otherpkg == pkg || !otherpkg->clientdata->fileslistvalid) continue;
+        if (otherpkg == divpkg) {
+          debug(dbg_eachfiledetail, "process_archive ... diverted, skipping\n");
+          continue;
+        }
 
         /* Found one.  We delete remove the list entry for this file,
          * (and any others in the same package) and then mark the package
@@ -890,7 +962,7 @@ void process_archive(const char *filename) {
   for (cfile= newfileslist; cfile; cfile= cfile->next) {
     if (cfile->namenode->flags & fnnf_new_conff) continue;
     fnametmpvb.used= fnameidlu;
-    varbufaddstr(&fnametmpvb,cfile->namenode->name);
+    varbufaddstr(&fnametmpvb,namenodetouse(cfile->namenode,pkg)->name);
     varbufaddstr(&fnametmpvb,DPKGTEMPEXT);
     varbufaddc(&fnametmpvb,0);
     ensure_pathname_nonexisting(fnametmpvb.buf);
